@@ -27,6 +27,11 @@ function create_kuryr_cache_dir {
     fi
 }
 
+function create_kuryr_lock_dir {
+    # Create lock directory
+    sudo install -d -o "$STACK_USER" "$KURYR_LOCK_DIR"
+}
+
 function get_distutils_data_path {
     cat << EOF | python -
 from __future__ import print_function
@@ -65,11 +70,24 @@ function configure_kuryr {
 
     iniset "$KURYR_CONFIG" kubernetes port_debug "$KURYR_PORT_DEBUG"
 
+    KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
+    if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
+        # This works around the issue of being unable to set oslo.privsep mode
+        # to FORK in os-vif. When running in a container we disable `sudo` that
+        # was prefixed before `privsep-helper` command. This let's us run in
+        # envs without sudo and keep the same python environment as the parent
+        # process.
+        iniset "$KURYR_CONFIG" vif_plug_ovs_privileged helper_command privsep-helper
+        iniset "$KURYR_CONFIG" vif_plug_linux_bridge_privileged helper_command privsep-helper
+    fi
+
     if is_service_enabled kuryr-daemon; then
         iniset "$KURYR_CONFIG" cni_daemon daemon_enabled True
-        KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
+        iniset "$KURYR_CONFIG" oslo_concurrency lock_path "$KURYR_LOCK_DIR"
+        create_kuryr_lock_dir
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
-            # When running kuryr-daemon in container we need to set up configs.
+            # When running kuryr-daemon in container we need to set up some
+            # configs.
             iniset "$KURYR_CONFIG" cni_daemon docker_mode True
             iniset "$KURYR_CONFIG" cni_daemon netns_proc_dir "/host_proc"
         fi
@@ -119,8 +137,7 @@ function generate_containerized_kuryr_resources {
     local output_dir="${DATA_DIR}/kuryr-kubernetes"
     generate_kuryr_configmap $output_dir $KURYR_CONFIG $KURYR_CNI_CONFIG
     generate_kuryr_service_account $output_dir
-    KURYR_USE_PORTS_POOLS=$(trueorfalse False KURYR_USE_PORTS_POOLS)
-    generate_controller_deployment $output_dir $KURYR_USE_PORTS_POOLS
+    generate_controller_deployment $output_dir $KURYR_HEALTH_SERVER_PORT
     generate_cni_daemon_set $output_dir $CNI_BIN_DIR $CNI_CONF_DIR
 }
 
@@ -491,13 +508,16 @@ function extract_hyperkube {
     tmp_loopback_cni_path="/tmp/loopback"
     tmp_nsenter_path="/tmp/nsenter"
 
-    hyperkube_container="$(docker ps -aq \
-        -f ancestor="${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" | \
-        head -1)"
+    hyperkube_container=$(docker run -d \
+        --net host \
+       "${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" \
+       /bin/false)
     docker cp "${hyperkube_container}:/hyperkube" "$tmp_hyperkube_path"
     docker cp "${hyperkube_container}:/opt/cni/bin/loopback" \
         "$tmp_loopback_cni_path"
     docker cp "${hyperkube_container}:/usr/bin/nsenter" "$tmp_nsenter_path"
+
+    docker rm "$hyperkube_container"
     sudo install -o "$STACK_USER" -m 0555 -D "$tmp_hyperkube_path" \
         "$KURYR_HYPERKUBE_BINARY"
     sudo install -o "$STACK_USER" -m 0555 -D "$tmp_loopback_cni_path" \
